=== modified file 'bzrlib/errors.py'
--- bzrlib/errors.py	2006-06-20 05:32:16 +0000
+++ bzrlib/errors.py	2006-07-10 23:00:44 +0000
@@ -965,6 +965,14 @@
     """A nested progress bar was not 'finished' correctly."""
 
 
+class InvalidProgressBarType(BzrNewError):
+    """Environment variable BZR_PROGRESS_BAR='%(bar_type)s is not a supported type
+Select one of: %(valid_types)s"""
+
+    def __init__(self, bar_type, valid_types):
+        BzrNewError.__init__(self, bar_type=bar_type, valid_types=valid_types)
+
+
 class UnsupportedOperation(BzrNewError):
     """The method %(mname)s is not supported on objects of type %(tname)s."""
     def __init__(self, method, method_self):

=== modified file 'bzrlib/progress.py'
--- bzrlib/progress.py	2006-07-10 21:24:55 +0000
+++ bzrlib/progress.py	2006-07-10 23:00:44 +0000
@@ -41,13 +41,14 @@
 import os
 
 import bzrlib.errors as errors
-from bzrlib.trace import mutter 
+from bzrlib.trace import mutter
 
 
 def _supports_progress(f):
-    if not hasattr(f, 'isatty'):
+    isatty = getattr(f, 'isatty', None)
+    if isatty is None:
         return False
-    if not f.isatty():
+    if not isatty():
         return False
     if os.environ.get('TERM') == 'dumb':
         # e.g. emacs compile window
@@ -55,16 +56,31 @@
     return True
 
 
+_progress_bar_types = {}
+
 
 def ProgressBar(to_file=None, **kwargs):
     """Abstract factory"""
     if to_file is None:
         to_file = sys.stderr
-    if _supports_progress(to_file):
-        return TTYProgressBar(to_file=to_file, **kwargs)
+    requested_bar_type = os.environ.get('BZR_PROGRESS_BAR')
+    # An value of '' or not set reverts to standard processing
+    if requested_bar_type in (None, ''):
+        if _supports_progress(to_file):
+            return TTYProgressBar(to_file=to_file, **kwargs)
+        else:
+            return DotsProgressBar(to_file=to_file, **kwargs)
     else:
-        return DotsProgressBar(to_file=to_file, **kwargs)
-    
+        # Minor sanitation to prevent spurious errors
+        requested_bar_type = requested_bar_type.lower().strip()
+        # TODO: jam 20060710 Arguably we shouldn't raise an exception
+        #       but should instead just disable progress bars if we
+        #       don't recognize the type
+        if requested_bar_type not in _progress_bar_types:
+            raise errors.InvalidProgressBarType(requested_bar_type,
+                                                _progress_bar_types.keys())
+        return _progress_bar_types[requested_bar_type](to_file=to_file, **kwargs)
+
  
 class ProgressBarStack(object):
     """A stack of progress bars."""
@@ -204,6 +220,10 @@
         return DummyProgress(**kwargs)
 
 
+_progress_bar_types['dummy'] = DummyProgress
+_progress_bar_types['none'] = DummyProgress
+
+
 class DotsProgressBar(_BaseProgressBar):
 
     def __init__(self, **kwargs):
@@ -231,6 +251,9 @@
     def child_update(self, message, current, total):
         self.tick()
 
+
+_progress_bar_types['dots'] = DotsProgressBar
+
     
 class TTYProgressBar(_BaseProgressBar):
     """Progress bar display object.
@@ -417,6 +440,9 @@
         #self.to_file.flush()        
 
 
+_progress_bar_types['tty'] = TTYProgressBar
+
+
 class ChildProgress(_BaseProgressBar):
     """A progress indicator that pushes its data to the parent"""
 

=== modified file 'bzrlib/tests/test_progress.py'
--- bzrlib/tests/test_progress.py	2006-07-10 21:50:56 +0000
+++ bzrlib/tests/test_progress.py	2006-07-10 23:00:44 +0000
@@ -17,6 +17,7 @@
 import os
 from StringIO import StringIO
 
+from bzrlib import errors
 from bzrlib.progress import (
         DummyProgress, ChildProgress,
         TTYProgressBar,
@@ -25,6 +26,7 @@
         )
 from bzrlib.tests import TestCase
 
+
 class FakeStack:
     def __init__(self, top):
         self.__top = top
@@ -275,3 +277,34 @@
         pb = self.get_nested(out, 'dumb')
         pb.finished()
         self.assertIsInstance(pb, DotsProgressBar)
+
+    def test_progress_env_tty(self):
+        # The environ variable BZR_PROGRESS_BAR controls what type of
+        # progress bar we will get, even if it wouldn't usually be that type
+        import cStringIO
+
+        # Usually, this would be a DotsProgressBar
+        out = cStringIO.StringIO()
+        pb = self.get_nested(out, 'dumb', 'tty')
+        pb.finished()
+        # Even though we are not a tty, the env_var will override
+        self.assertIsInstance(pb, TTYProgressBar)
+
+    def test_progress_env_dots(self):
+        # Even though we are in a tty, the env_var will override
+        out = _TTYStringIO()
+        pb = self.get_nested(out, 'xterm', 'dots')
+        pb.finished()
+        self.assertIsInstance(pb, DotsProgressBar)
+
+    def test_progress_env_none(self):
+        # Even though we are in a valid tty, no progress
+        out = _TTYStringIO()
+        pb = self.get_nested(out, 'xterm', 'none')
+        pb.finished()
+        self.assertIsInstance(pb, DummyProgress)
+
+    def test_progress_env_invalid(self):
+        out = _TTYStringIO()
+        self.assertRaises(errors.InvalidProgressBarType, self.get_nested,
+            out, 'xterm', 'nonexistant')

